
subroutine CalcEnergy(Pos, L, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: eps = 4.9, sigma = 2.43, B = 5.6/eps, r0 = 2.46/sigma, del = 1.09/sigma 
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: Shift, rc2, d, g, gaussian, d2, id2, id6, id12
    integer :: i, j
    PEnergy = 0.
    Shift = -4.*(rc**(-12)-rc**(-6)) 
    rc2 = rc*rc
    do i = 0, NAtom-1
        Posi = Pos(i,:)
		do j = i+1, NAtom-1
			rij = Pos(j,:)-Posi
			rij = rij - L* dnint(rij/L) 
			d2 = sum(rij*rij)
			d = sqrt(d2)
			if (d2 > rc2) then
				cycle
			end if
			id2 = 1./d2    !this takes into account the sigma so the rest of the ds are dimensionless
			id6 = id2*id2*id2
			id12 = id6 * id6
			g = (d-r0)/del
			gaussian = B*exp(-g*g)
			PEnergy = PEnergy + 4.*(id12-id6) + Shift + gaussian
		enddo
    enddo 
end subroutine


subroutine CalcEnergyForces(Pos, L, rc, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
	!f2py intent(in, out) :: Forces
    real(8), parameter :: eps = 4.9, sigma = 2.43, B = 5.6/eps, r0 = 2.46/sigma, del = 1.09/sigma 
     real(8), dimension(Dim) :: rij, Fij, Posi
    real(8) :: Shift, rc2, d, g, gaussian, d2, id2, id6, id12
    integer :: i, j
    PEnergy = 0.
	Forces = 0.
    Shift = -4.*(rc**(-12)-rc**(-6))
    rc2 = rc*rc
    do i = 0, NAtom-1
        Posi = Pos(i,:)
		do j = i+1, NAtom-1
			rij = Pos(j,:)-Posi
			rij = rij - L* dnint(rij/L)
			d2 = sum(rij*rij)
			d = sqrt(d2)
			if (d2 > rc2) then
				cycle
			end if
			id2 = 1./d2   !this takes into account the sigma so the rest of the ds are dimensionless
			id6 = id2*id2*id2
			id12 = id6 * id6
			g = (d-r0)/del
			gaussian = B*exp(-g*g)
			PEnergy = PEnergy + 4.*(id12-id6) + Shift + gaussian
			Fij = rij*(24.*(-2.*id12+id6)*id2-2*gaussian*g/(del*d))
			Forces(i,:)=Forces(i,:) + Fij
			Forces(j,:)=Forces(j,:) - Fij
		enddo
    enddo 
end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, L, rc, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in) :: L, rc, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel 
    Vel = Vel + 0.5 * dt * Accel 
    call CalcEnergyForces(Pos, L, rc, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel 
    KEnergy = 0.5 * sum(Vel*Vel) 
end subroutine