subroutine CalcEnergy(Pos, L, Y, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, Y
    real(8), intent(out) :: PEnergy
    real(8), parameter :: eps = 20.38, sigma = 2.429, B = 23.35/eps, r0 = 2.465/sigma, del = 1.090/sigma, epsW = 0.8943, sigmaW = 3.448
    real(8), dimension(Dim) :: rij, rij_w1, rij_w2, Posi, Posw1, Posw2
    real(8) :: d, g, gaussian, d2, id2, id6, id12, d2_w1, d2_w2, d_w1, d_w2, id3_w1, id3_w2, id9_w1, id9_w2
    integer :: i, j
    PEnergy = 0.
    do i = 0, NAtom-1
        Posi = Pos(i,:)
        !first calculate PE due to interaction with the walls
        !find a wall molecule closest to the one you're looking at (x,y are the same and z is different, and equal to L/2)
        Posw1 = Posi
        Posw2 = Posi
        Posw1(2) = -Y/2.0
        Posw2(2) = Y/2.0
        rij_w1 = Posw1 - Posi
        rij_w2 = Posw2 - Posi
        
        d2_w1= sum(rij_w1*rij_w1)
        d_w1= sqrt(d2_w1)
        id3_w1 = 1./(d2_w1*d_w1)
        id9_w1 = id3_w1*id3_w1*id3_w1
        
        d2_w2= sum(rij_w2*rij_w2)
        d_w2= sqrt(d2_w2)
        id3_w2 = 1./(d2_w2*d_w2)
        id9_w2 = id3_w2*id3_w2*id3_w2
        
        PEnergy = epsW/eps*(sigmaW/sigma)**3.*(4.*3.14/3.)*(1./15.*(sigmaW/sigma)**6.*id9_w1-1./2.*id3_w1)
        PEnergy = PEnergy + epsW/eps*(sigmaW/sigma)**3.*(4.*3.14/3.)*(1./15.*(sigmaW/sigma)**6.*id9_w2-1./2.*id3_w2)
        
        do j = i+1, NAtom-1
            rij = Pos(j,:)-Posi
			rij = rij - L* dnint(rij/L) 
            d2 = sum(rij*rij)
            d = sqrt(d2)
            id2 = 1./d2
            id6 = id2*id2*id2
            id12 = id6 * id6
            g = (d-r0)/del
            gaussian = B*exp(-g*g)
            PEnergy = PEnergy + 4.*(id12-id6) + gaussian
        enddo
     enddo
end subroutine

subroutine CalcEnergyForces(Pos, L, Y, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, Y
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
    !f2py intent(in, out) :: Forces
    real(8), parameter :: eps = 20.38, sigma = 2.429, B = 23.35/eps, r0 = 2.465/sigma, del = 1.090/sigma, epsW = 0.8943, sigmaW = 3.448
    real(8), dimension(Dim) :: rij, Fij, Fiw1, Fiw2, Fiw, rij_w1, rij_w2, Posi, Posw1, Posw2
    real(8) :: d, g, gaussian, d2, id2, id6, id12, d2_w1, d2_w2, d_w1, d_w2, id3_w1, id3_w2, id9_w1, id9_w2
    integer :: i, j
    PEnergy = 0.
    Forces = 0.
    do i = 0, NAtom-1
        Posi = Pos(i,:)
        !first calculate force due to interaction with the walls
        !find a wall molecule closest to the one you're looking at (x,y are the same and z is different, and equal to L/2)
        Posw1 = Posi
        Posw2 = Posi
        Posw1(2) = -Y/2.0
        Posw2(2) = Y/2.0
        rij_w1 = Posw1 - Posi
        rij_w2 = Posw2 - Posi
        
        d2_w1= sum(rij_w1*rij_w1)
        d_w1= sqrt(d2_w1)
        id3_w1 = 1./(d2_w1*d_w1)
        id9_w1 = id3_w1*id3_w1*id3_w1
        
        d2_w2= sum(rij_w2*rij_w2)
        d_w2= sqrt(d2_w2)
        id3_w2 = 1./(d2_w2*d_w2)
        id9_w2 = id3_w2*id3_w2*id3_w2
        
        Fiw1 = rij_w1* epsW/eps*(sigmaW/sigma)**5.*(4.*3.14)*(-3./15.*(sigmaW/sigma)**6.*id9_w1+1./2.*id3_w1)/d2_w1
        Fiw2 = rij_w2* epsW/eps*(sigmaW/sigma)**5.*(4.*3.14)*(-3./15.*(sigmaW/sigma)**6.*id9_w2+1./2.*id3_w2)/d2_w2
        
        Fiw = Fiw1 + Fiw2
        Forces(i,:)=Forces(i,:) + Fiw
        
        do j = i+1, NAtom-1
            rij = Pos(j,:)-Posi
			rij = rij - L* dnint(rij/L) 
            d2 = sum(rij*rij)
            d = sqrt(d2)
            id2 = 1./d2
            id6 = id2*id2*id2
            id12 = id6 * id6
            g = (d-r0)/del
            gaussian = B*exp(-g*g)
            PEnergy = PEnergy + 4.*(id12-id6) + gaussian
            Fij = rij*(24.*(-2.*id12+id6)*id2-2*gaussian*g/(del*d))
            Forces(i,:)=Forces(i,:) + Fij
            Forces(j,:)=Forces(j,:) - Fij
        enddo
    enddo 
end subroutine

subroutine VVIntegrate(Pos, Vel, Accel, L, Y, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in) :: L, Y, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel 
    Vel = Vel + 0.5 * dt * Accel 
    call CalcEnergyForces(Pos, L, Y, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel 
    KEnergy = 0.5 * sum(Vel*Vel) 
end subroutine